@@ -210,8 +210,8 @@ f.procedures2str <- function(procs)
       psi <- lproc$args$psi
       ## test if tuning.psi is the default one
       if (!is.null(lproc$args$tuning.psi) &&
-          isTRUE(all.equal(lproc$args$tuning.psi, f.lmrob.control(psi = psi)$tuning.psi)))
-        psi <- paste(psi,lproc$args$tuning.psi)
+	  isTRUE(all.equal(lproc$args$tuning.psi, .Mpsi.tuning.default(psi))))
+	psi <- paste(psi, lproc$args$tuning.psi)
     }
     D.type <- switch(lproc$estname,
                      lmrob.u =,
@@ -1237,14 +1237,14 @@ f.prediction.points <- function(design, type = c('pc', 'grid'),
              ## calculate distances to boundaries and take the minimal one
              lmin <- which.min(sapply(lfct, function(x) sum((rpc$loadings[,id] * x)^2)))
              ## create sequence of multiplicands
-             lmult <- seq(0,lfct[lmin],length.out=length.out/NCOL(rpc$loadings))
+             lmult <- seq(0,lfct[lmin], length.out=length.out/NCOL(rpc$loadings))
              rdf <- rbind(rdf, rep(rob$center,each=length(lmult)-1) +
                           direction*lmult[-1] %*% t(rpc$loadings[,id]))
            }
          },
          grid = {
            ## generate sequences for every dimension
-           lval <- as.data.frame(apply(lrange,2,f.seq,
+           lval <- as.data.frame(apply(lrange, 2L, f.seq,
                                        length.out = round(length.out^(1/NCOL(design))) ))
            ## return if 1 dimension, otherwise create all combinations
            rdf <- if (NCOL(design) > 1)
